﻿using System;
using System.Web.Mvc;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Text;
using System.Text.RegularExpressions;
using Telerik.Web.UI;
using Telerik.Web.UI.Common;
using Telerik.Web.Mvc.UI;


namespace Telerik.Web.Mvc.Extensions
{
	public static class RadControlsHtmlHelpers
	{
		private static int idCounter = 0;
		public static string RadControlCss<T>(this HtmlHelper htmlHelper, Action<T> initializeControl)
			where T : WebControl, ISkinnableControl, new()
		{
			var control = CreateControl(initializeControl);
			control.EnableEmbeddedBaseStylesheet = true;
			control.EnableEmbeddedSkins = true;

			return ControlRenderer.GetControlCssReferences(control);
		}

		public static string RadControl<T>(this HtmlHelper htmlHelper, Action<T> initializeControl)
			where T : WebControl, ISkinnableControl, new()
		{
			return ControlRenderer.GetControlHtml(CreateControl(initializeControl));
		}

		private static T CreateControl<T>(Action<T> initializeControl) where T : WebControl, ISkinnableControl, new()
		{
			var id = "RadControl" + (++idCounter).ToString();
			var control = new T
			{
				ID = id,
				RegisterWithScriptManager = false,
				EnableEmbeddedSkins = false,
				EnableEmbeddedBaseStylesheet = false,
				EnableEmbeddedScripts = false
			};
			var page = new Page();
			page.Controls.Add(control);

			initializeControl(control);
			if (control is RadTimePicker)
            {
                
                var picker = (control as RadTimePicker);
                SetStyleClasses(picker.TimeView);
                picker.TimeView.DataBind();
            }

            if (control is RadDateTimePicker)
            {
                var picker = (control as RadDateTimePicker);
                var timeView = (picker.TimeView as RadTimeView);
                timeView.DataBind();
                SetStyleClasses(timeView);
            }

			return control;
		}

		public static string RadControlScripts<T>(this HtmlHelper htmlHelper, Action<T> initializeControl)
			where T : WebControl, ISkinnableControl, new()
		{

			var control = CreateControl(initializeControl);
			control.EnableEmbeddedScripts = true;
			return ControlRenderer.GetControlScripts(control);
		}

		public static string RadControlScripts<T>(this HtmlHelper htmlHelper)
			where T : WebControl, ISkinnableControl, new()
		{
			return RadControlScripts<T>(htmlHelper, delegate { });
		}

		public static string RadControlCss<T>(this HtmlHelper htmlHelper) where T : WebControl, ISkinnableControl, new()
		{
			return RadControlCss<T>(htmlHelper, delegate { });
		}

		public static string RadControl<T>(this HtmlHelper htmlHelper)
			where T : WebControl, ISkinnableControl, new()
		{
			return RadControl<T>(htmlHelper, delegate { });
		}

        private static void SetStyleClasses(RadTimeView view)
        {
            view.TimeStyle.CssClass = FormatCssClass(view, "", view.TimeStyle.CssClass);
            view.AlternatingTimeStyle.CssClass = FormatCssClass(view, "", view.AlternatingTimeStyle.CssClass);
            view.HeaderStyle.CssClass = FormatCssClass(view, "rcHeader", view.HeaderStyle.CssClass);
            view.FooterStyle.CssClass = FormatCssClass(view, "rcFooter", view.FooterStyle.CssClass);
            view.TimeOverStyle.CssClass = FormatCssClass(view, "rcHover", view.TimeOverStyle.CssClass);
            view.CssClass = FormatCssClass(view, "RadCalendarTimeView", view.CssClass);
        }

        private static string FormatCssClass(RadTimeView view, string prefix, string userDefined)
        {
            string skinCssClass = "";
            if (prefix == "RadCalendarTimeView")
            {
                skinCssClass = String.IsNullOrEmpty(view.Skin) ? prefix : String.Format("{0} {0}_{1}", prefix, view.Skin);
            }
            else
            {
                skinCssClass = prefix;
            }
            userDefined = Regex.Replace(userDefined, prefix + "_\\S+\\s?", "");

            if (userDefined.IndexOf(skinCssClass) >= 0)
                return userDefined;
            else
                if (userDefined == String.Empty)
                    return skinCssClass;
                else
                    return String.Format("{0} {1}", skinCssClass, userDefined);
        }
	}
}
